/**
 * Copyright 2014 Internet2
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @author mchyzer
 * $Id: XmlExportGroup.java 6216 2010-01-10 04:52:30Z mchyzer $
 */
package edu.internet2.middleware.grouper.xml.export;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

import org.apache.commons.logging.Log;
import org.dom4j.Element;
import org.dom4j.ElementHandler;
import org.dom4j.ElementPath;
import org.hibernate.Query;
import org.hibernate.ScrollableResults;
import org.hibernate.Session;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import com.thoughtworks.xstream.io.xml.CompactWriter;
import com.thoughtworks.xstream.io.xml.Dom4JReader;

import edu.internet2.middleware.grouper.Composite;
import edu.internet2.middleware.grouper.hibernate.AuditControl;
import edu.internet2.middleware.grouper.hibernate.GrouperTransactionType;
import edu.internet2.middleware.grouper.hibernate.HibUtils;
import edu.internet2.middleware.grouper.hibernate.HibernateHandler;
import edu.internet2.middleware.grouper.hibernate.HibernateHandlerBean;
import edu.internet2.middleware.grouper.hibernate.HibernateSession;
import edu.internet2.middleware.grouper.internal.dao.GrouperDAOException;
import edu.internet2.middleware.grouper.misc.CompositeType;
import edu.internet2.middleware.grouper.misc.GrouperVersion;
import edu.internet2.middleware.grouper.util.GrouperUtil;
import edu.internet2.middleware.grouper.xml.importXml.XmlImportMain;


/**
 *
 */
public class XmlExportComposite {

  /**
   * 
   */
  private static final String XML_EXPORT_COMPOSITE_XPATH = "/grouperExport/composites/XmlExportComposite";

  /**
   * 
   */
  private static final String COMPOSITES_XPATH = "/grouperExport/composites";

  /** uuid */
  private String uuid;
  
  /** creatorId */
  private String creatorId;

  /** createTime */
  private String createTime;

  /** hibernateVersionNumber */
  private long hibernateVersionNumber;

  /** contextId */
  private String contextId;
  
  /** owner */
  private String owner;
  
  /** leftFactor */
  private String leftFactor;
  
  /** rightFactor */
  private String rightFactor;
  
  /** type */
  private String type;

  /**
   * logger 
   */
  private static final Log LOG = GrouperUtil.getLog(XmlExportComposite.class);

  /**
   * owner
   * @return owner
   */
  public String getOwner() {
    return this.owner;
  }

  /**
   * owner
   * @param owner1
   */
  public void setOwner(String owner1) {
    this.owner = owner1;
  }

  /**
   * left factor
   * @return left factor
   */
  public String getLeftFactor() {
    return this.leftFactor;
  }

  /**
   * left factor
   * @param leftFactor1
   */
  public void setLeftFactor(String leftFactor1) {
    this.leftFactor = leftFactor1;
  }

  /**
   * right factor
   * @return right factor
   */
  public String getRightFactor() {
    return this.rightFactor;
  }

  /**
   * right factor
   * @param rightFactor1
   */
  public void setRightFactor(String rightFactor1) {
    this.rightFactor = rightFactor1;
  }

  /**
   * type
   * @return type
   */
  public String getType() {
    return this.type;
  }

  /**
   * type
   * @param type1
   */
  public void setType(String type1) {
    this.type = type1;
  }

  /**
   * 
   */
  public XmlExportComposite() {
    
  }

  /**
   * uuid
   * @return uuid
   */
  public String getUuid() {
    return this.uuid;
  }

  /**
   * uuid
   * @param uuid1
   */
  public void setUuid(String uuid1) {
    this.uuid = uuid1;
  }

  /**
   * creatorId
   * @return creatorId
   */
  public String getCreatorId() {
    return this.creatorId;
  }

  /**
   * creatorId
   * @param creatorId1
   */
  public void setCreatorId(String creatorId1) {
    this.creatorId = creatorId1;
  }

  /**
   * createTime
   * @return createTime
   */
  public String getCreateTime() {
    return this.createTime;
  }

  /**
   * createTime
   * @param createTime1
   */
  public void setCreateTime(String createTime1) {
    this.createTime = createTime1;
  }

  /**
   * hibernateVersionNumber
   * @return hibernateVersionNumber
   */
  public long getHibernateVersionNumber() {
    return this.hibernateVersionNumber;
  }

  /**
   * hibernateVersionNumber
   * @param hibernateVersionNumber1
   */
  public void setHibernateVersionNumber(long hibernateVersionNumber1) {
    this.hibernateVersionNumber = hibernateVersionNumber1;
  }

  /**
   * contextId
   * @return contextId
   */
  public String getContextId() {
    return this.contextId;
  }

  /**
   * contextId
   * @param contextId1
   */
  public void setContextId(String contextId1) {
    this.contextId = contextId1;
  }
  
  /**
   * convert to composite
   * @return the composite
   */
  public Composite toComposite() {
    Composite composite = new Composite();
    
    composite.setContextId(this.contextId);
    composite.setCreateTime(GrouperUtil.defaultIfNull(GrouperUtil.dateLongValue(this.createTime), 0L));
    composite.setCreatorUuid(this.creatorId);
    composite.setHibernateVersionNumber(this.hibernateVersionNumber);
    composite.setLeftFactorUuid(this.leftFactor);
    composite.setFactorOwnerUuid(this.owner);
    composite.setRightFactorUuid(this.rightFactor);
    composite.setTypeDb(this.type);
    composite.setUuid(this.uuid);
    
    return composite;
  }

  /**
   * @param exportVersion
   * @return the xml string
   */
  public String toXml(GrouperVersion exportVersion) {
    StringWriter stringWriter = new StringWriter();
    this.toXml(exportVersion, stringWriter);
    return stringWriter.toString();
  }

  /**
   * @param exportVersion 
   * @param writer
   */
  public void toXml(
      @SuppressWarnings("unused") GrouperVersion exportVersion, Writer writer) {
    XStream xStream = XmlExportUtils.xstream();
  
    CompactWriter compactWriter = new CompactWriter(writer);
    
    xStream.marshal(this, compactWriter);
  
  }

  /**
   * parse the xml file for groups
   * @param xmlImportMain
   */
  public static void processXmlSecondPass(final XmlImportMain xmlImportMain) {
    xmlImportMain.getReader().addHandler( COMPOSITES_XPATH, 
        new ElementHandler() {
            public void onStart(ElementPath path) {
            }
            public void onEnd(ElementPath path) {
                // process a ROW element
                Element row = path.getCurrent();
  
                // prune the tree
                row.detach();
            }
        }
    );
  
    xmlImportMain.getReader().addHandler( XML_EXPORT_COMPOSITE_XPATH, 
        new ElementHandler() {
            public void onStart(ElementPath path) {
                // do nothing here...    
            }
            public void onEnd(ElementPath path) {

              Element row = null;
              try {
                // process a ROW element
                row = path.getCurrent();
  
                // prune the tree
                row.detach();
  
                XmlExportComposite xmlExportCompositeFromFile = (XmlExportComposite)xmlImportMain.getXstream().unmarshal(new Dom4JReader(row));
                
                Composite composite = xmlExportCompositeFromFile.toComposite();
                
                XmlExportUtils.syncImportable(composite, xmlImportMain);
                
                xmlImportMain.incrementCurrentCount();
              } catch (RuntimeException re) {
                LOG.error("Problem importing composite: " + XmlExportUtils.toString(row), re);
                throw re;
              }
            }
        }
    );
  
  }

  /**
   * get db count
   * @param xmlExportMain 
   * @return db count
   */
  public static long dbCount(XmlExportMain xmlExportMain) {
    long result = HibernateSession.byHqlStatic().createQuery("select count(theComposite) " 
        + exportFromOnQuery(xmlExportMain, false)).uniqueResult(Long.class);
    return result;
  }
  
  /**
   * get the query from the FROM clause on to the end for export
   * @param xmlExportMain
   * @param includeOrderBy 
   * @return the export query
   */
  private static String exportFromOnQuery(XmlExportMain xmlExportMain, boolean includeOrderBy) {
    //select all members in order
    StringBuilder queryBuilder = new StringBuilder();
    if (!xmlExportMain.filterStemsOrObjects()) {
      queryBuilder.append(" from Composite as theComposite ");
    } else {
      queryBuilder.append(
          " from Composite as theComposite where exists ( select theGroup from Group as theGroup "
          + " where theComposite.factorOwnerUuid = theGroup.uuid and ( ");
      xmlExportMain.appendHqlStemLikeOrObjectEquals(queryBuilder, "theGroup", "nameDb", false);
      queryBuilder.append(" ) ) "
          + " and exists ( select theGroup from Group as theGroup "
          + " where theComposite.leftFactorUuid = theGroup.uuid and ( ");
      xmlExportMain.appendHqlStemLikeOrObjectEquals(queryBuilder, "theGroup", "nameDb", false);
      queryBuilder.append(" ) ) "
          + " and exists ( select theGroup from Group as theGroup "
          + " where theComposite.rightFactorUuid = theGroup.uuid and ( ");
      xmlExportMain.appendHqlStemLikeOrObjectEquals(queryBuilder, "theGroup", "nameDb", false);
      queryBuilder.append(" ) ) ");
    }
    if  (includeOrderBy) {
      queryBuilder.append(" order by theComposite.factorOwnerUuid, theComposite.leftFactorUuid, " +
        "theComposite.rightFactorUuid, theComposite.typeDb ");
      
    }
    return queryBuilder.toString();
  }

  /**
   * 
   * @param writer
   * @param xmlExportMain
   */
  public static void exportCompositesGsh(final Writer writer, final XmlExportMain xmlExportMain) {
    //get the members
    HibernateSession.callbackHibernateSession(GrouperTransactionType.READONLY_OR_USE_EXISTING, AuditControl.WILL_NOT_AUDIT, new HibernateHandler() {
      
      public Object callback(HibernateHandlerBean hibernateHandlerBean)
          throws GrouperDAOException {

        Session session = hibernateHandlerBean.getHibernateSession().getSession();

        //select all members in order
        Query query = session.createQuery(
            "select distinct "
            + " ( select theGroup.nameDb from Group theGroup where theGroup.uuid = theComposite.factorOwnerUuid ), "
            + " theComposite.typeDb, "
            + " ( select theGroup.nameDb from Group theGroup where theGroup.uuid = theComposite.leftFactorUuid ), "
            + " ( select theGroup.nameDb from Group theGroup where theGroup.uuid = theComposite.rightFactorUuid )"
            + exportFromOnQuery(xmlExportMain, false));


        final GrouperVersion grouperVersion = new GrouperVersion(GrouperVersion.GROUPER_VERSION);

        //this is an efficient low-memory way to iterate through a resultset
        ScrollableResults results = null;
        try {
          results = query.scroll();
          while(results.next()) {
            final String overallGroupName = (String)results.get(0);
            final String type = (String)results.get(1);
            final String leftFactorName = (String)results.get(2);
            final String rightFactorName = (String)results.get(3);

            //writer.write("" + subjectId + ", " + sourceId + ", " + listName + ", " + groupName 
            //    + ", " + stemName + ", " + nameOfAttributeDef 
            //    + ", " + enabledTime + ", " + disabledTime  + "\n");
            
            HibernateSession.callbackHibernateSession(GrouperTransactionType.READONLY_NEW, AuditControl.WILL_NOT_AUDIT, new HibernateHandler() {
              
              public Object callback(HibernateHandlerBean hibernateHandlerBean)
                  throws GrouperDAOException {
                try {
                  XmlExportComposite.toGsh(grouperVersion, writer, overallGroupName, type, leftFactorName, rightFactorName);
                } catch (IOException ioe) {
                  throw new RuntimeException("Problem exporting composite to gsh: " + overallGroupName, ioe);
                }
                return null;
              }
            });
            xmlExportMain.incrementRecordCount();
          }
        } finally {
          HibUtils.closeQuietly(results);
        }
          
        return null;
      }
    });
  }

  /**
   * convert this to GSH that is failsafe
   * @param grouperVersion
   * @param writer
   * @param ownerGroupName 
   * @param type 
   * @param leftFactorName 
   * @param rightFactorName 
   * @throws IOException 
   */
  public static void toGsh(GrouperVersion grouperVersion, Writer writer, String ownerGroupName, 
      String type, String leftFactorName, String rightFactorName) throws IOException {

    writer.write("Group ownerGroup = GroupFinder.findByName(grouperSession, \""
        + GrouperUtil.escapeDoubleQuotesSlashesAndNewlinesForString(ownerGroupName) + "\", false);\n");
    writer.write("Group leftFactorGroup = GroupFinder.findByName(grouperSession, \""
        + GrouperUtil.escapeDoubleQuotesSlashesAndNewlinesForString(leftFactorName) + "\", false);\n");
    writer.write("Group rightFactorGroup = GroupFinder.findByName(grouperSession, \""
        + GrouperUtil.escapeDoubleQuotesSlashesAndNewlinesForString(rightFactorName) + "\", false);\n");

    writer.write("CompositeType compositeType = CompositeType." + CompositeType.valueOfIgnoreCase(type).name() + ";\n");
    
    writer.write("if (ownerGroup != null) { ");

    writer.write("if (leftFactorGroup != null) { ");

    writer.write("if (rightFactorGroup != null) { ");

    //addCompositeMember(CompositeType type, Group left, Group right)
    writer.write(" CompositeSave compositeSave = new CompositeSave(grouperSession).assignOwnerGroup(ownerGroup).assignCompositeType(compositeType)"
        + ".assignLeftFactorGroup(leftFactorGroup).assignRightFactorGroup(rightFactorGroup); gshTotalObjectCount++; Composite composite = compositeSave.save(); "
        + "if (compositeSave.getSaveResultType() != SaveResultType.NO_CHANGE) {System.out.println(\"Made change for composite: \" + composite.toString()); "
        + "gshTotalChangeCount++;} ");

    writer.write(" } else { System.out.println(\"ERROR: cant find rightFactorGroup: '" + GrouperUtil.escapeDoubleQuotesSlashesAndNewlinesForString(rightFactorName) + "'\"); gshTotalErrorCount++; } ");

    writer.write(" } else { System.out.println(\"ERROR: cant find leftFactorGroup: '" + GrouperUtil.escapeDoubleQuotesSlashesAndNewlinesForString(leftFactorName) + "'\"); gshTotalErrorCount++; } ");

    writer.write(" } else { System.out.println(\"ERROR: cant find overallGroup: '" + GrouperUtil.escapeDoubleQuotesSlashesAndNewlinesForString(ownerGroupName) + "'\"); gshTotalErrorCount++; }\n");

  }

  /**
   * 
   * @param writer
   * @param xmlExportMain
   */
  public static void exportComposites(final Writer writer, final XmlExportMain xmlExportMain) {
    //get the members
    HibernateSession.callbackHibernateSession(GrouperTransactionType.READONLY_OR_USE_EXISTING, AuditControl.WILL_NOT_AUDIT, new HibernateHandler() {
      
      public Object callback(HibernateHandlerBean hibernateHandlerBean)
          throws GrouperDAOException {
  
        Session session = hibernateHandlerBean.getHibernateSession().getSession();
  
        //select all members in order
        Query query = session.createQuery(
            "select distinct theComposite " + exportFromOnQuery(xmlExportMain, true));
  
        GrouperVersion grouperVersion = new GrouperVersion(GrouperVersion.GROUPER_VERSION);
        try {
          writer.write("  <composites>\n");
  
          //this is an efficient low-memory way to iterate through a resultset
          ScrollableResults results = null;
          try {
            results = query.scroll();
            while(results.next()) {
              Object object = results.get(0);
              final Composite composite = (Composite)object;

              //comments to dereference the foreign keys
              if (xmlExportMain.isIncludeComments()) {
                HibernateSession.callbackHibernateSession(GrouperTransactionType.READONLY_NEW, AuditControl.WILL_NOT_AUDIT, new HibernateHandler() {
                  
                  public Object callback(HibernateHandlerBean hibernateHandlerBean)
                      throws GrouperDAOException {
                    try {
                      writer.write("\n    <!-- ");
                      XmlExportUtils.toStringComposite(writer, composite, false);
                      writer.write(" -->\n");
                      return null;
                    } catch (IOException ioe) {
                      throw new RuntimeException(ioe);
                    }
                  }
                });
              }
              XmlExportComposite xmlExportComposite = composite.xmlToExportComposite(grouperVersion);
              writer.write("    ");
              xmlExportComposite.toXml(grouperVersion, writer);
              writer.write("\n");
              xmlExportMain.incrementRecordCount();
            }
          } finally {
            HibUtils.closeQuietly(results);
          }
          
          
          if (xmlExportMain.isIncludeComments()) {
            writer.write("\n");
          }
          
          //end the members element 
          writer.write("  </composites>\n");
        } catch (IOException ioe) {
          throw new RuntimeException("Problem with streaming composites", ioe);
        }
        return null;
      }
    });
  }

  /**
   * take a reader (e.g. dom reader) and convert to an xml export group
   * @param exportVersion
   * @param hierarchicalStreamReader
   * @return the bean
   */
  public static XmlExportComposite fromXml(@SuppressWarnings("unused") GrouperVersion exportVersion, 
      HierarchicalStreamReader hierarchicalStreamReader) {
    XStream xStream = XmlExportUtils.xstream();
    
    XmlExportComposite xmlExportGroupType = (XmlExportComposite)xStream.unmarshal(hierarchicalStreamReader);
  
    return xmlExportGroupType;
  }

  /**
   * 
   * @param exportVersion
   * @param xml
   * @return the object from xml
   */
  public static XmlExportComposite fromXml(
      @SuppressWarnings("unused") GrouperVersion exportVersion, String xml) {
    XStream xStream = XmlExportUtils.xstream();
    
    XmlExportComposite xmlExportGroupType = (XmlExportComposite)xStream.fromXML(xml);
  
    return xmlExportGroupType;
  }

  /**
   * parse the xml file for composites
   * @param xmlImportMain
   */
  public static void processXmlFirstPass(final XmlImportMain xmlImportMain) {
    xmlImportMain.getReader().addHandler( COMPOSITES_XPATH, 
        new ElementHandler() {
            public void onStart(ElementPath path) {
            }
            public void onEnd(ElementPath path) {
                // process a ROW element
                Element row = path.getCurrent();

                // prune the tree
                row.detach();
            }
        }
    );

    xmlImportMain.getReader().addHandler( XML_EXPORT_COMPOSITE_XPATH, 
        new ElementHandler() {
            public void onStart(ElementPath path) {
                // do nothing here...    
            }
            public void onEnd(ElementPath path) {
                // process a ROW element
                Element row = path.getCurrent();

                // prune the tree
                row.detach();

                xmlImportMain.incrementTotalImportFileCount();
            }
        }
    );
 
  }
}
